#! /bin/env python

import copy
import carmatable as ct
import scipy.interpolate as interp
import drpacs

def time2time(time):
    """convert an 'hh:mm:ss.s' array into a float array of seconds since midnight"""
    ftime = []
    for t in time:
        w = t.split(':')
        ftime.append( float(w[0])*3600.0 + float(w[1])*60.0 + float(w[2]))
    return ftime

def baseLinePhaseCalcHelper(baselinesInput,phaseInput):
    """
    
    """
    fin=open(phaseInput)
    baselinesTable=fin.readlines()
    header=baselinesTable[3].split()[2:]
    times=[line.split()[0] for line in baselinesTable[4:]]
    baselinesTable=[line.split()[1:] for line in baselinesTable[4:]]
    ants={}
    for ant,phases in zip(header,list(zip(*baselinesTable))):
        ants[int(ant)]=[int(x) for x in phases]
    return ants,times



def baseLinePhaseCalc(baselinesInput, cPhaseInput, sPhaseInput, cFoutput, sFoutput, interpOutput, doAUnWrap=True, doBUnWrap=False, factor=-1):
    """
    This replaces an earlier version that zipped the files and assumed 's' and 'c' on same timeframe
    This version interpolates as well
    Input: baselinesInput
           cPhaseInput
           sPhaseInput
           doAUnWrap     antenna based unwrapping
           doBUnWrap     baseline based unwrapping
           factor        scale factor cFreq/sFreq for expected phase coherence, used for unwrapping
    Output:   cFoutput      
              sFoutput
              interpOutput
    """
    def getbl(tableIn, colname):
        """
           parse a column a-b from a table into an list of ant pairs [(a,b),...]
        """
        blString = tableIn.getCol(colname)
        bl = []
        for bs in blString:
            a,b=bs.split('-')
            bl.append((int(a),int(b)))
        return blString,bl
    def keepnearzero(phases, report=False):
        """keep an array of phases on average near 0, wrapping could have messed that up
           only allows a single offset accross the whole timeline
        """
        n = 0
        mean = 0
        np = len(phases)
        for p in phases:
            mean = mean + np
        mean = mean / np
        if mean > 180:
            if report: print "mean %f too high" % mean
            for i in range(0,np):
                phases[i] = phases[i] - 360
            n = 1
        elif mean < -180:
            if report: print "mean %f too low" % mean
            for i in range(0,np):
                phases[i] = phases[i] + 360
            n = 1
        return n
    def keepnearfactor(cp,sp,f):
        """keep it close enough:  keep sp*factor - cp small
           this is heavily based on the fact that sp[] doesn't wrap,
           but because factor is large, cp can easily wrap
        """
        n=0
        m=0
        for i in range(0,len(cp)):
            pp=f*sp[i]
            if cp[i] > pp+180:
                cp[i] = cp[i] - 360
                m=m+1
            elif cp[i] < pp-180:
                cp[i] = cp[i] + 360
                m=m+1
            n=n+1
        return (n,m)
        

    #
    tableIn=ct.CarmaTable()
    tableIn.open(baselinesInput)
    cBaselinesString,cBaselines=getbl(tableIn,"CARMA")
    sBaselinesString,sBaselines=getbl(tableIn,"SZA")
    
    #  
    cants,ctimes=baseLinePhaseCalcHelper(baselinesInput,cPhaseInput)
    sants,stimes=baseLinePhaseCalcHelper(baselinesInput,sPhaseInput)
    ny = 0
    nz = 0
    if doAUnWrap:
        print "Unwrapping antenna based phases from CARMA/SZA log files"
        for a in range(1,len(cants)+1):
            cants[a] = drpacs.unwrap(cants[a])
            ny = ny + 1
            nz = nz + keepnearzero(cants[a])
        for a in range(1,len(sants)+1):
            sants[a] = drpacs.unwrap(sants[a])
            ny = ny + 1
            nz = nz + keepnearzero(sants[a],True)
    # CARMA
    cOutput=ct.CarmaTable(["Time"]+cBaselinesString,["hms"]+len(cBaselinesString)*["i"],[""]*(1+len(cBaselinesString)))
    cOutput.editCol(ctimes,0)
    if doBUnWrap:
        print "Unwrapping baseline based phases from CARMA/SZA log files"
        for ant1,ant2 in cBaselines:
            phases = [a-b for a,b in zip(cants[ant1],cants[ant2])]
            phases = drpacs.unwrap(phases)
            ny = ny + 1
            nz = nz + keepnearzero(phases)
            cOutput.editCol(phases,"%i-%i"%(ant1,ant2))
    else:
        for ant1,ant2 in cBaselines:
            cOutput.editCol([a-b for a,b in zip(cants[ant1],cants[ant2])],"%i-%i"%(ant1,ant2))
    cOutput.write(cFoutput)

    # SZA
    sOutput=ct.CarmaTable(["Time"]+sBaselinesString,["hms"]+len(sBaselinesString)*["i"],[""]*(1+len(sBaselinesString)))
    sOutput.editCol(stimes,0)
    if doBUnWrap:
        for ant1,ant2 in sBaselines:
            phases = [a-b for a,b in zip(sants[ant1],sants[ant2])]
            phases =drpacs.unwrap(phases)
            ny = ny + 1
            nz = nz + keepnearzero(phases,True)
            sOutput.editCol(phases,"%i-%i"%(ant1,ant2))
    else:
        for ant1,ant2 in sBaselines:
            sOutput.editCol([a-b for a,b in zip(sants[ant1],sants[ant2])],"%i-%i"%(ant1,ant2))
    sOutput.write(sFoutput)

    print "Found %d/%d baselines were keepnearzero re-aligned during unwrap" % (nz,ny)
    
    
    # SZA interpolation
    # ..there is still a bug in here, are there assumptions about the timerange?
    # ..need to take the common times really to guarentee success
    sInterpOutput=ct.CarmaTable(sOutput.getColNames(),sOutput.getColTypes(),sOutput.getColUnits())
    sInterpOutput.editCol(ctimes,"Time")
    for baseline in sOutput.getColNames()[1:]:
        func=interp.interp1d(sOutput.getDMSCol("Time"),sOutput.getIntCol(baseline),bounds_error=False)
        sInterpOutput.editCol(func(sInterpOutput.getDMSCol("Time")),baseline)
    for row in range(sInterpOutput.getNrows()-1,-1,-1):# Going backwards means that we will not overflow the newly shorter table.
        if 'nan' in sInterpOutput.getRow(row):
            sInterpOutput.removeRow(row)
    sInterpOutput.write(interpOutput)

    # unwrapping but keeping CARMA on the SZA wrap
    if factor>0:
        show=True
        work=0
        tmp1=cOutput.getCol(0)
        tmp2=sInterpOutput.getCol(0)
        if len(tmp1) != len(tmp2):
            print "Bad news, CAR and SZA_interp do not have the same size...."
            print tmp1[0],tmp2[0]
            print tmp1[1],tmp2[1]
            print tmp1[2],tmp2[2]
            print "..."
            print tmp1[len(tmp1)-1],tmp2[len(tmp2)-1]
            if len(tmp1) == len(tmp2)+1:
                print "FIXING by removing first row from CARMA"
                cOutput.removeRow(0)
            else:
                print "NO FIX AVAILABLE YET?"
        n=0
        m=0
        for c,s in zip(cBaselines, sBaselines):
            c1 = c[0]
            c2 = c[1]
            s1 = s[0]
            s2 = s[1]
            sPh = [a-b for a,b in zip(sants[s1],sants[s2])]
            cPh = [a-b for a,b in zip(cants[c1],cants[c2])]
            cPh1 = cOutput.getIntCol("%i-%i"%(c1,c2))
            sPh1 = sInterpOutput.getFloatCol("%i-%i"%(s1,s2))
            if show:
                print 'Testing array lengths:',len(cPh),len(sPh),' -> ',len(cPh1),len(sPh1)
                if len(cPh1) != len(sPh1):
                    print "...that's bad, tryng to fix..."
                show = False
            if len(cPh1) == len(sPh1):
                (n1,m1) = keepnearfactor(cPh1,sPh1,factor)
                cOutput.editCol(cPh1,"%i-%i"%(c1,c2))
                work = work + 1
                n=n+n1
                m=m+m1
        if work>0:
            print "Writing re-aligned CARMA phases based on freqRatio=%f" % factor
            print "Found %d/%d points were re-aligned" % (m,n)
            cOutput.write(cFoutput)                
                
